// SetupFile.h | by Doug Manley
// SetupFile is a collection of functions to aid loading and parsing txt setup files.
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
#ifndef SETUPFILE_H
#define SETUPFILE_H
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
#include <StringExtras\StringExtras.h>
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
string SF_ReadALineFrom( FILE* f );

void SF_CutTrailingSpaces( string &str );

void SF_CutLeadingSpaces( string &str );

void SF_Separate( string str, string &strField,string &strParam );
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
/* This function reads a line from an open FILE* and cuts of the
   newline character ('\n'). */
string SF_ReadALineFrom( FILE* f ) {
	char cstr[1024];
	fgets( cstr,1024,f );
	if( feof(f) ) {
		return("");
	}
	cstr[strlen(cstr)-1] = '\0';
	return(cstr);
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
/* This functions removes all non-characters from the end of a
   String.
*/
void SF_CutTrailingSpaces( string &str ) {
	int length = str.length();
	int pos = length-1;
	while( true ) {
		if( pos >= 0 && pos < length && !IsCharacter(str[pos]) ) {
			pos--;
		} else {
			break;
		}
	}
	//printf("strLen: %i ; pos: %i\n",length,pos );
	str = str.substr( 0,pos+1 );
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
/* This function removes all non-characters from the beginning of
   a String.
*/
void SF_CutLeadingSpaces( string &str ) {
	int pos = 0;
	while( true ) {
		if( pos >= str.length() ) {
			break;
		}
		if( IsCharacter(str[pos]) ) {
			break;
		}
		pos++;
	}
	str = str.substr( pos,-1 );
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
/* This function separates a line from the file into its
   'field' and 'parameter' values.
*/
void SF_Separate( string str, string &strField,string &strParam ) {
	int pos = str.find( "::" );

	strField = str.substr(0,pos);
		//printf("strField (unprocessed): >>%s<<\n",strField.c_str() );
	SF_CutLeadingSpaces( strField );
		//printf("strField (no leading): >>%s<<\n",strField.c_str() );
	SF_CutTrailingSpaces( strField );
		//printf("strField (no trailing): >>%s<<\n",strField.c_str() );


	if( pos == -1 ) {
		strParam = strField;
		return;
	}

	strParam = str.substr(pos+2,-1);
		//printf("strParam (unprocessed): >>%s<<\n",strParam.c_str() );
	SF_CutLeadingSpaces( strParam );
		//printf("strParam (no leading): >>%s<<\n",strParam.c_str() );
	SF_CutTrailingSpaces( strParam );
		//printf("strParam (no trailing): >>%s<<\n",strParam.c_str() );
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
class SetupFile {
public:
	SetupFile() : sf_line(""), sf_field(""), sf_parameter(""), sf_file(NULL), sf_comment("//") {
	};
	~SetupFile() {
		Close();
	};

	void SetComment( string c ) {  sf_comment = c;  }

	bool Open( char *cstr ) {
		Close();
		sf_file = fopen( cstr, "rt" );
		if( sf_file ) {
			return(true);
		} else {
			return(false);
		}
	};
	bool Close() {
		if( sf_file ) {
			fclose( sf_file );
			return(true);
		} else {
			return(false);
		}
	};

	bool ReadLine() {
		// Read the line from the file:
		sf_line = SF_ReadALineFrom( sf_file );
		// Filter out any comments:
		string::size_type pos = sf_line.find( sf_comment );
		if( pos != (string::size_type)-1 ) {
			sf_line = sf_line.substr( 0,pos );
		}
		// Break it into parameters:
		SF_Separate( sf_line, sf_field,sf_parameter );

		// Returns:
		if( feof(sf_file) && !StringsAreEqual_IgnoreCase(sf_line,"") ) {
			return(false);
		} else {
			return(true);
		}
	};

	bool FieldIs( string str ) {
		return( StringsAreEqual_IgnoreCase( sf_field,str ) );
	};
	
	string Parameter() {
		return( sf_parameter );
	};

	bool eof() {
		return( feof(sf_file) != 0 );
	};
private:
	string sf_line;
	string sf_field, sf_parameter;
	FILE *sf_file;
	string sf_comment;
};
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////////////